﻿using Borepin.Base;
using Prism.Commands;
using Prism.Navigation;
using System.Threading.Tasks;
using System.Windows.Input;
using FabAccessAPI.Schema;
using Borepin.Model;
using Prism.Services;
using Borepin.Service;
using Borepin.Base.Exceptions;
using Capnp.Rpc;
using System;
using Borepin.ViewModel;
using System.Collections.Generic;
using NaturalSort.Extension;
using System.Linq;

namespace Borepin.PageModel
{
    public class UserPageModel : ConnectionModelBase
    {
        #region Private Fields
        private string _ID;
        private User _User;
        #endregion

        #region Contructors
        public UserPageModel(INavigationService navigationService, IPageDialogService pageDialogService, IAPIService apiService) : base(navigationService, pageDialogService, apiService)
        {
            
        }
        #endregion

        #region Data
        public override Task LoadInstance(object instance)
        {
            if(instance is string)
            {
                _ID = instance as string;
            }
            else
            {
                throw new InstanceIncorrectException();
            }

            return Task.CompletedTask;
        }

        public override async Task LoadAPIData()
        {
            _User = (await _API.Session.UserSystem.Search.GetUserByName(_ID).ConfigureAwait(false)).Just;
            UserItem = new UserVisualize(_User);
            UserItem.LoadData();

            IReadOnlyList<Role> role_list = await _API.Session.PermissionSystem.Info.GetRoleList().ConfigureAwait(false);
            List<Role> user_role_list = new List<Role>(await _User.Info.ListRoles().ConfigureAwait(false));

            List<PermissionSelectViewModel> viewmodel_list = new List<PermissionSelectViewModel>();
            foreach (Role role in role_list)
            {
                PermissionSelectViewModel new_viewmodel = new PermissionSelectViewModel(_NavigationService, _PageDialogService, _APIService);
                object[] array = new object[]
                {
                    role,
                    _User,
                    user_role_list.Contains(role)
                };

                await new_viewmodel.LoadInstance(array).ConfigureAwait(false);
                viewmodel_list.Add(new_viewmodel);
            }

            viewmodel_list.OrderBy(x => x.RoleName, StringComparison.OrdinalIgnoreCase.WithNaturalSort());
            PermissionSelectViewModel_List = viewmodel_list;
        }
        #endregion

        #region Fields
        private IList<PermissionSelectViewModel> _PermissionSelectViewModel_List;
        public IList<PermissionSelectViewModel> PermissionSelectViewModel_List
        {
            get => _PermissionSelectViewModel_List;
            set => SetProperty(ref _PermissionSelectViewModel_List, value);
        }

        private UserVisualize _UserItem;
        public UserVisualize UserItem
        {
            get => _UserItem;
            set => SetProperty(ref _UserItem, value);
        }
        #endregion

        #region Commands
        
        #endregion
    }
}
